function txt=help_from_scicosblock(funname,path)
// Build a help file from the .sci source file of a Scilab function. 
//
// Calling Sequence
//  txt = help_from_scicosblock(funname[,path])
//  txt = help_from_scicosblock(dirname[,path])
//
// Parameters
//  funname: the name of .sci source file to be processed.
//  dirname: directory name where all .sci files will be processed.
//  path: optional path where the xml help file will be created.
//  txt: returns the XML code if path is empty, or the path to the new .xml file. 
//
// Description
//  help_from_scicosblock is an advanced version of the Scilab function help_skeleton.
//  <literal>txt = help_from_scicosblock(funname)</literal> generates XML code for a help file
//  based on the first comment section in the file funname.sci.
//  
//  <literal>fullpath = help_skeleton(funname,path)</literal> generates XML code with 
//  the help text for the function <literal>funname</literal>. The XML code is stored in 
//  file named <literal>funname.xml</literal> in the directory <literal>path</literal>. 
// 
//  The content of the help file is based on the first continuous comment section in the 
//  scilab source file. This format of this section should follow some simple rules:
//
//  The first comment line following the function definition should contain a short description 
//  of the function.
//
//  The remaining comments are formated according to the following (optional) headlines:
//  "Calling Sequence", "Parameters", "Description", "Examples", "See also", "Used functions", 
//  "Authors" and "Bibliography".
//
//  The following formating guidelines should be used when writing the source code comments:
//
//  <literal>Parameters</literal> - separate parameter name and description by a ":". 
//  Keep the description of each parameter on the same line. 
//
//  <literal>Description</literal> - formating of the text can be done using XML commands.
//  See the source code for help_from_scicosblock for examples. 
//  
//  <literal>See also</literal> - write one function name on each line following the See also headline.  
//
//  <literal>Authors</literal> - write one author on each line following the Authors headline. 
//  Use ";" to separate the authors name (appears in bold) from any add additional information.
//
//  <literal>References</literal> - write one reference on each line following the References headline.  
//
// Examples
//  help_from_scicosblock()   // start without arguments to generate a source code template.
//                    // Edit the template and save it as e.g test_fun.sci in the current directory.
//  help_from_scicosblock('test_fun')        // return the xml skeleton as a text string
//  help_from_scicosblock('test_fun','.')    // create the xml help file in the current directory.
//  xmltohtml('.',[],'html.xsl','html') // create the file test_fun.html in current directory.
//  See also the initial comment section of help_from_scicosblock.sci for more examples.
//
//
// Authors
//  T. Pettersen ; torbjorn.pettersen@broadpark.no


  //	
  //    Copyright (C) 2008  Torbjorn Pettersen
  //		      2009  Holger Nahrstaedt		
  //		
  // This program is free software; you can redistribute it and/or modify
  // it under the terms of the GNU General Public License as published by
  // the Free Software Foundation; either version 2 of the License, or
  // (at your option) any later version.
  // 
  // This program is distributed in the hope that it will be useful,
  // but WITHOUT ANY WARRANTY; without even the implied warranty of
  // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  // GNU General Public License for more details.
  // 
  // You should have received a copy of the GNU General Public License
  // along with this program; if not, write to the Free Software
  // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

if argn(2)==0 then
  template=[
    'function [z]=function_template(x,y)'
    '// Short description on the first line following the function header.'
    '//'
    '// Block Screenshot'
    '//  ' 
    '//'
    '//'
    '// Description'
    '// Here is a description of the function.'
    '// Add an empty comment line to format the text into separate paragraphs.'
    '//'
    '// XML format commands may also be used directly in the text.'
    '// See the source code for help_from_scicosblock.sci for more examples on how to write the initial comment section.'
    '//'
    '// Dialog box'
    '//'
    '// Default properties'
    '//  '
    '//  '
    '//'
    '// Interfacing function'
    '//  '
    '//  '
    '// Computational function'
    '//  '
    '//  '
    '// Authors'
    '//  Authors names ; should be listed one pr line. Use "";"" to separate names from additional information '
    ''
    '// start of code'
    'z = sin(x).*cos(x+y);'
    'endfunction'
  ];
//   f=mopen(TMPDIR+'/function_template.sci','w'); mfprintf(f,'%s\n',template); mclose(f);
//   scipad(TMPDIR+'/function_template.sci');
  return;
end

if isdir(funname) then
  printf('\nReading from directory %s \n',funname);
  files=dir(funname+'\*.sci');
  files=files(2);
  for i=1:size(files,'r'),
    if argn(2)==2 then
      help_from_scicosblock(files(i),path);
      printf(' ...reading %s/%s.sci ... writing %s/%s.xml\n',funname,files(i),path,files(i));
    else
      help_from_scicosblock(files(i),'.');
      printf(' ...reading %s/%s.sci ... writing %s.xml\n',funname,files(i),files(i));
    end
  end
  txt=''; return;
  printf('help_from_scicosblock processed %i files.\n',i);
end

out=tokens(funname,'\'); out=out($);  // windows
out=tokens(out,'/'); out=out($);      // linux/unix
out=tokens(out,'.'); out=out(1);   // remove .sci (...wont work for fil.name.sci)

txt=[
'<?xml version=""1.0"" encoding=""ISO-8859-1""?>'
'<!--'
' * This help file was generated from '+out+'.sci using help_from_scicosblock().'
' * '
' *     Copyright (C) '+date()+' Author of '+out+'.sci'
' * '
' *  This program is free software; you can redistribute it and/or modify'
' *  it under the terms of the GNU General Public License as published by'
' *  the Free Software Foundation; either version 2 of the License, or'
' *  (at your option) any later version.'
' *  '
' *  This program is distributed in the hope that it will be useful,'
' *  but WITHOUT ANY WARRANTY; without even the implied warranty of'
' *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the'
' *  GNU General Public License for more details.'
' *  '
' *  You should have received a copy of the GNU General Public License'
' *  along with this program; if not, write to the Free Software'
' *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA'
' * '
' -->'
'<refentry version=""5.0-subset Scilab"" xml:id=""'+out+'"" xml:lang=""en""'
'          xmlns=""http://docbook.org/ns/docbook""'
'          xmlns:xlink=""http://www.w3.org/1999/xlink""'
'          xmlns:svg=""http://www.w3.org/2000/svg""'
'          xmlns:ns3=""http://www.w3.org/1999/xhtml""'
'          xmlns:mml=""http://www.w3.org/1998/Math/MathML""'
'          xmlns:db=""http://docbook.org/ns/docbook"">'
'  <info>'
'    <pubdate>$LastChangedDate: '+date()+' $</pubdate>'
'  </info>'
''
]

if isempty(strindex(funname,'.sci')) then funname=funname+'.sci'; end;
if isempty(fileinfo(funname)) then error(funname+' does not exists'); end;
f=mopen(funname,'r');
if isempty(f) then error('Failed to read '+funname+'.sci'); end
line=' '; doc=[]; 

while isempty(strindex(line,'function ')) & ~meof(f), line=mgetl(f,1); end 
line=mgetl(f,1); short_descr=stripblanks(strsubst(line,'//',''),%T);
txt=[txt;
'  <refnamediv>'
'    <refname>'+out+'</refname><refpurpose>'+short_descr+'</refpurpose>'
'  </refnamediv>'
''];

cmds=['BLOCK SCREENSHOT','DESCRIPTION','DIALOG BOX','DEFAULT PROPERTIES','INTERFACING FUNCTION',..
      'COMPUTATIONAL FUNCTION','AUTHORS'];

doing='search'; i=strindex(line,'//');
line=mgetl(f,1); 
while (~isempty(stripblanks(line)) & ~meof(f)),
  if stripblanks(line)=='//' then 
    if doing=='Description' then 
      in='new_descr_param'; 
    else
      in='';
    end
  else 
    in=strsplit(line,i(1)+1); 
    in=stripblanks(in(2)); 
  end

  IN=convstr(in,'u');
  if find(cmds==IN) then
    [add_txt,doing]=change_activity(doing,in)
    txt=[txt;add_txt];
	// consider to remove used headers from cmds in order to avoid repetitive use.
  else
    if doing=='Block Screenshot' then
      txt=[txt;];
    elseif doing=='Description' & in=='new_descr_param' then
      txt=[txt;'   </para>';'   <para>'];
    elseif doing=='Description' then
      txt=[txt;in];
    elseif doing=='Dialog box' then
      i=strindex(in,':'); 
      if ~isempty(i) then
        if length(in)>i(1) then
          in=strsplit(in,i(1)); par_name=in(1); par_descr=in(2);
        else
          par_name=in; par_descr=' ';
        end
        txt=[txt;'  </listitem> <listitem><para><emphasis role='"bold'">'+par_name+'</emphasis></para>'];
        txt=[txt;'       <para>'+par_descr+'</para>'];
      else
	 txt=[txt;'   <para>'+in+'</para>'];
      end
    elseif doing=='Default Properties' then
      i=strindex(in,':'); 
      if ~isempty(i) then
        if length(in)>i(1) then
          in=strsplit(in,i(1)); par_name=in(1); par_descr=in(2);
        else
          par_name=in; par_descr=' ';
        end
         txt=[txt;'      <listitem><para><emphasis role='"bold'">'+par_name+'</emphasis>'+par_descr+'</para></listitem>'];
      end
    elseif doing=='Interfacing Function' then
      txt=[txt;'   <para> '+in+' </para> '];
    elseif doing=='Computational Function' then
      txt=[txt;'    <para> '+in+' </para> '];
    elseif doing=='Authors' & convstr(in,'u')~='AUTHORS' & ~isempty(stripblanks(in)) then
      [name,ref]=chop(in,';');
      if isempty(ref) then
        txt=[txt;'   <varlistentry><term>'+name+'</term></varlistentry>'];
      else
        txt=[txt;'   <varlistentry><term>'+name+'</term><listitem><para>'+ref+'</para></listitem></varlistentry>'];
      end
    end
  end
  line=mgetl(f,1); i=strindex(line,'//');
end
txt=[txt;change_activity(doing,'FINISHED')];
mclose(f);
if argn(2)==2 then
  mputl(txt,pathconvert(path,%t,%f)+out+'.xml')
  txt=pathconvert(path,%t,%f)+out+'.xml'
end
endfunction

////////////////////////////////////////////////////////////////////////
function [head,tail]=chop(str,tok)
  i=regexp(str,'/'+tok+'/','o');
  if isempty(i) then
    head=str;
    tail=[];
  else
    head=part(str,1:i-1);
    tail=part(str,i+1:length(str));
  end  
endfunction

////////////////////////////////////////////////////////////////////////
function [txt,doing]=change_activity(currently_doing,start_doing)
  doing=start_doing; 
  select convstr(currently_doing,"u")
    case 'BLOCK SCREENSHOT' then
      txt=['</refsection>'];
    case 'DESCRIPTION' then
      txt=['</para>';'</refsection>'];
    case 'DIALOG BOX' then
      txt=['   </listitem>';'  </itemizedlist>';'</refsection>'];
    case 'DEFAULT PROPERTIES' then
      txt=['  </itemizedlist>';'</refsection>'];      
    case 'INTERFACING FUNCTION' then
      txt=['   </listitem>';'  </itemizedlist>';'</refsection>'];
    case 'COMPUTATIONAL FUNCTION' then
      txt=['   </listitem>';'  </itemizedlist>';'</refsection>'];
    case 'AUTHORS' then
      txt=['   </variablelist>';'</refsection>'];
    else
      txt='';
  end    

  select convstr(start_doing,"u"),
      case 'BLOCK SCREENSHOT'
      txt=[txt;'';'<refsection>';'   <title>Block Screenshot</title>'];
    case 'DESCRIPTION'
      txt=[txt;'';'<refsection>';'   <title>Description</title>';'   <para>'];
    case 'DIALOG BOX'
      txt=[txt;'';'<refsection>';'   <title>Dialog box</title>';'       <itemizedlist>';'         <listitem>' ];
    case 'DEFAULT PROPERTIES'
      txt=[txt;'';'<refsection>';'   <title>Default Properties</title>';'       <itemizedlist>'];
    case 'INTERFACING FUNCTION'
      txt=[txt;'';'<refsection>';'   <title>Interfacing Function</title>';'       <itemizedlist>';'         <listitem>' ];
    case 'COMPUTATIONAL FUNCTION'
      txt=[txt;'';'<refsection>';'   <title>Computational Function</title>';'   <itemizedlist>';'         <listitem>'];
    case 'AUTHORS'
      txt=[txt;'';'<refsection>';'   <title>Authors</title>';'   <variablelist>'];
    case 'FINISHED'
      txt=[txt;'</refentry>'];
  end
endfunction
