
/**-----------------------------------------------------------------------------------
 *  Copyright (C) 2006
 *  Copyright (C) 2008  Holger Nahrstaedt
 *
 *  This file is part of HART, the Hardware Access in Real Time Toolbox for Scilab/Scicos.
 *
 *  HART is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  HART is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with HART; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *--------------------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "iir_filter.h"


/*
  Runs a filter forwards and backwards over the data. No attempt is made
  to deal with the edje effects or with the filter history. It is assumed
  that long pieces of data will be filtered and the edjes discarded.
*/
int mIIRfiltfilt(miirfilter *filt, double *indata, double *outdata, int ndata)
{
  int     j;
  double *tmp1, *tmp2;

  tmp1 = (double*)calloc(ndata, sizeof(double));
  if(tmp1==NULL)
  {
    fprintf(stderr, "### mIIRfiltfilt: error allocating tmp1 vector.\n");
    return -1;
  }
  tmp2 = (double*)calloc(ndata, sizeof(double));
  if(tmp2==NULL)
  {
    fprintf(stderr, "### mIIRfiltfilt: error allocating tmp2 vector.\n");
    return -1;
  }


  // forward filter
  mIIRfilter(filt, indata, tmp1, ndata);

  // reverse output data
  for(j=0; j<ndata; j++)
    tmp2[j] = tmp1[ndata-1-j];

  // now backward filter
  mIIRfilter(filt, tmp2, tmp1, ndata);

  // reverse output data
  for(j=0; j<ndata; j++)
    outdata[j] = tmp1[ndata-1-j];

  free(tmp1);
  free(tmp2);

  return 0;
}

/*!
 *  This function applies an IIR filter to the data
 *  and fills the output data vector with the result.
 *  The data vectors are assumed to be big enough to
 *  perform the operation. The history values of the
 *  filter are filled in automatically.
 *
 *  \retval 0
 */
int mIIRfilter(miirfilter *filt, double *indata, double *outdata, int ndata)
{
  int     i, tap, N;
  double  *a, *b;
  double  *hi, *ho;
  double   g;
  double  *p;

  //printf("*** filtering data with %s...\n", filt->name);

  N = filt->ntaps - 1;
  a = &(filt->a[0]);
  b = &(filt->b[0]);
  hi = &(filt->histin[0]);
  ho = &(filt->histout[0]);
  g = filt->gain;

  /* make sure B[0] = 0.0; */
  b[0] = 0.0;
  p = &(outdata[0]);

  /* filter the first few samples that involve history values */
  if(g==1)
  {
    for(i=0; i<N && i<ndata; i++)
    {
      *p = 0.0;
      for(tap=0; tap<=N; tap++)
      {
        if((i-tap) >= 0 && (i-tap) < ndata)
        {
          *p += a[tap]*indata[i-tap];
          *p -= b[tap]*outdata[i-tap];
        }
        else
        {
          *p += a[tap]*hi[N-tap+i];
          *p -= b[tap]*ho[N-tap+i];
        }
      }
      p++;
    }

    /* now filter the rest   */
    for(i=N; i<ndata; i++)
    {
      *p = 0.0;
      for(tap=0; tap<=N; tap++)
      {
        *p += a[tap]*indata[i-tap];
        *p -= b[tap]*outdata[i-tap];
      }
      p++;
    }
  }
  else
  {
    for(i=0; i<N && i<ndata; i++)
    {

      *p = 0.0;
      for(tap=0; tap<=N; tap++)
      {
        if((i-tap) >= 0 && (i-tap) < ndata)
        {
          *p += g*a[tap]*indata[i-tap];
          *p -= b[tap]*outdata[i-tap];
        }
        else
        {
          *p += g*a[tap]*hi[N-tap+i];
          *p -= b[tap]*ho[N-tap+i];
        }
      }
      p++;
    }

    /* now filter the rest   */
    for(i=N; i<ndata; i++)
    {
      *p = 0.0;
      for(tap=0; tap<=N; tap++)
      {
        *p += g*a[tap]*indata[i-tap];
        *p -= b[tap]*outdata[i-tap];
      }
      p++;
    }
  }

  /* store history values from the end of both input and output streams */
  for(i=0; i<N; i++)
  {
    if(N<=ndata)
    {
      hi[i] = indata[ndata-N+i];
      ho[i] = outdata[ndata-N+i];
    }
    else
    {
      if(i<ndata)
      {
        hi[i] = hi[i+1];
        ho[i] = ho[i+1];
      }
      else
      {
        hi[i] = indata[ndata-N+i];
        ho[i] = outdata[ndata-N+i];
      }
    }
  }

  b[0] = 1.0;
  //printf("*** filtering data...done\n");

  return 0;
}



/*
 *  This function applies an adaptive IIR filter to the data
 *  and fills the output data vector with the result.
 *  The data vectors are assumed to be big enough to
 *  perform the operation. The history values of the
 *  filter are filled in automatically.
 *
 *  \retval 0
 */
int mIIRfilterAdapt(miirfilterAdapt *filt, double *indata, double *outdata, int ndata)
{
  int     i, tap, N;
  double  asum, bsum;

  //printf("*** filtering data with %s...\n", filt->name);

  N = filt->ntaps - 1;

  /* make sure B[0] = 0.0; */
  for(i=1; i<MAXFS;i++)
    filt->b[0][i] = 0.0;

  /* filter the first few samples that involve history values */
  for(i=0; i<N; i++)
  {
    outdata[i] = asum = bsum = 0.0;
    for(tap=0; tap<=N; tap++)
    {
      if((i-tap) >= 0)
      {
        asum += filt->gain*filt->a[tap][i]*indata[i-tap];
        bsum += filt->b[tap][i]*outdata[i-tap];
      }
      else
      {
        asum += filt->gain*filt->a[tap][i]*filt->histin[N-tap+i];
        bsum += filt->b[tap][i]*filt->histout[N-tap+i];
      }
    }
    outdata[i] = asum - bsum;
  }

  /* now filter the rest   */
  for(i=N; i<ndata; i++)
  {
    asum = bsum = outdata[i] = 0.0;
    for(tap=0; tap<=N; tap++)
    {
      asum += filt->gain*filt->a[tap][i]*indata[i-tap];
      bsum += filt->b[tap][i]*outdata[i-tap];
    }
    outdata[i] = asum - bsum;
  }

  /* store history values from the end of both input and output streams */
  for(i=0; i<N; i++)
  {
    filt->histin[i] = indata[ndata-N+i];
    filt->histout[i] = outdata[ndata-N+i];
  }

  for(i=0; i<MAXFS;i++)
    filt->b[0][i] = 1.0;
  //printf("*** filtering data...done\n");

  return 0;
}



/*!
    Return a complex pole filter designed using the bilinear
    transform.

    The filter coefficients are defined by:

	  \f[ a_0 = 1/k  \f]
	  \f[ a_1 = -2/k  \f]
	  \f[ a_2 = -1/k  \f]
	  \f[ b_0 = 1  \f]
	  \f[ b_1 = \frac{2\omega_0^2 - 8f_s^2}{k\omega_0^2}  \f]
	  \f[ b_2 = \frac{q\omega_0^2 + 4qf_s^2 - 2\omega_0f_s}{kq\omega_0^2}  \f]

    where

    \f[ k = \frac{q\omega_0^2+4qf_s^2+2\omega_0f_s}{q\omega_0^2} \f]


    \param	f0  The pole frequency (Hz)
    \param  q  The pole Q
    \param  fs  The sample frequency of the filter
    \param  filt  A pointer to an miirfilter

    \retval 0
 */
int bilinear_complex_pole(float f0, float q, int fs, miirfilter *filt)
{
  double w0, w02, k;
  double a[3], b[3];

  w0 = f0*2*M_PIl;
  w02 = w0*w0;

  k = (q*w02 + 4.0*q*fs*fs + 2.0*w0*fs) / (q*w02);

  /* Define coefficients */
  b[0] = 1;
  b[1] = (2.0*w02-8.0*fs*fs) / (k*w02);
  b[2] = (q*w02 + 4.0*q*fs*fs - 2.0*w0*fs) / (k*q*w02);

  a[0] = 1.0/k * -2.0;
  a[1] = -2.0/k * -2.0;
  a[2] = -1.0/k * -2.0;

  /* build filter */

  sprintf(filt->name, "bcp%3.2f-%d", f0, fs);
  filt->fs = fs;
  filt->ntaps = 3;
  filt->dcgain = iirdcgain(a, b, filt->ntaps);
  filt->hfgain = iirhfgain(a, b, filt->ntaps);
  filt->gain = 1.0;

  filt->a[0] = a[0];
  filt->a[1] = a[1];
  filt->a[2] = a[2];
  filt->b[0] = b[0];
  filt->b[1] = b[1];
  filt->b[2] = b[2];

  return 0;
}


/*!
    Return a complex zero filter designed using the bilinear
    transform.

    The filter coefficients are defined by:

    \f[ a_0 = \frac{-Q\omega_0^2/2 - 2Qf_s^2 - \omega_0f_s}{Q\omega_0^2} \f]
    \f[ a_1 = \frac{-\omega_0^2 + 4f_s^2}{\omega_0^2} \f]
    \f[ a_2 = \frac{-Q\omega_0^2/2 - 2Qf_s^2 + \omega_0f_s}{Q\omega_0^2} \f]

    \f[ b_0 = 1.0 \f]
    \f[ b_1 = -2.0 \f]
    \f[ b_2 = -1.0 \f]


    \param	f0  The zero frequency (Hz)
    \param  q  The zero Q
    \param  fs  The sample frequency of the filter
    \param  filt  A pointer to an miirfilter

    \retval 0
 */
int bilinear_complex_zero(float f0, float q, int fs, miirfilter *filt)
{
  double w0, w02;
  double a[3], b[3];

  w0 = f0*2*M_PIl;
  w02 = w0*w0;

  /* Define coefficients */
  a[0] = (-q*w02/2.0 - 2.0*q*fs*fs - w0*fs) / (q*w02);
  a[1] = (-w02+4.0*fs*fs) / w02;
  a[2] = (-q*w02/2.0 - 2.0*q*fs*fs + w0*fs) / (q*w02);

  b[0] = 1.0;
  b[1] = -2.0;
  b[2] = -1.0;

  /* build filter */

  sprintf(filt->name, "bcz%3.2f-%d", f0, fs);
  filt->fs = fs;
  filt->ntaps = 3;
  filt->dcgain = iirdcgain(a, b, filt->ntaps);
  filt->hfgain = iirhfgain(a, b, filt->ntaps);
  filt->gain = 1.0;

  filt->a[0] = a[0];
  filt->a[1] = a[1];
  filt->a[2] = a[2];
  filt->b[0] = b[0];
  filt->b[1] = b[1];
  filt->b[2] = b[2];

  return 0;
}

/*!
    Return a real pole filter designed using the bilinear
    transform.

    The filter coefficients are defined by:

    \f[ a_0 = \frac{\omega_0}{2f_s + \omega_0} \f]
    \f[ a_1 = a_0 \f]

    \f[ b_0 = 1.0 \f]
    \f[ b_1 = \frac{\omega_0-2f_s}{\omega_0+2f_s} \f]


    \param	f0  The pole frequency (Hz)
    \param  fs  The sample frequency of the filter
    \param  filt  A pointer to an miirfilter

    \retval 0
 */
int bilinear_real_pole(float f0, int fs, miirfilter *filt)
{
  double w0;
  double a[2], b[2];

  w0 = f0*2*M_PIl;

  /* Define coefficients */
  a[0] = w0 / (2.0*fs + w0);
  a[1] = a[0];

  b[0] = 1.0;
  b[1] = (w0-2.0*fs) / (w0+2.0*fs);


  /* build filter */

  sprintf(filt->name, "brp%3.2f-%d", f0, fs);
  filt->fs = fs;
  filt->ntaps = 2;
  filt->dcgain = iirdcgain(a, b, filt->ntaps);
  filt->hfgain = iirhfgain(a, b, filt->ntaps);
  filt->gain = 1.0;

  filt->a[0] = a[0];
  filt->a[1] = a[1];
  filt->b[0] = b[0];
  filt->b[1] = b[1];

  return 0;
}

/*!
    Return a real zero filter designed using the bilinear
    transform.

    The filter coefficients are defined by:

    \f[ a_0 = \frac{2f_s + \omega_0}{\omega_0} \f]
    \f[ a_1 = \frac{-2f_s + \omega_0}{\omega_0} \f]

    \f[ b_0 = 1.0 \f]
    \f[ b_1 = 1.0 \f]

    \param	f0  The zero frequency (Hz)
    \param  fs  The sample frequency of the filter
    \param  filt  A pointer to an miirfilter

    \retval 0
 */
int bilinear_real_zero(float f0, int fs, miirfilter *filt)
{
  double w0;
  double a[2], b[2];

  w0 = f0*2*M_PIl;

  /* Define coefficients */
  a[0] = (2.0*fs + w0) / w0;
  a[1] = (-2.0*fs + w0) / w0;

  b[0] = 1.0;
  b[1] = 1.0;


  /* build filter */

  sprintf(filt->name, "brz%3.2f-%d", f0, fs);
  filt->fs = fs;
  filt->ntaps = 2;
  filt->dcgain = iirdcgain(a, b, filt->ntaps);
  filt->hfgain = iirhfgain(a, b, filt->ntaps);
  filt->gain = 1.0;

  filt->a[0] = a[0];
  filt->a[1] = a[1];
  filt->b[0] = b[0];
  filt->b[1] = b[1];

  return 0;
}



/*!
    Return a complex pole and zero filter designed using the bilinear
    transform.

    The filter coefficients are defined by:


    \f[ a_0 = \frac{4f_s^2 + 2\omega_zf_s/Q_z + \omega_z^2}{k} \f]
    \f[ a_1 = \frac{2\omega_z^2 - 8f_s^2}{k} \f]
    \f[ a_2 = \frac{4f_s^2 - 2\omega_zf_s/Q_z + \omega_z^2}{k} \f]
    \f[ b_0 = 1.0 \f]
    \f[ b_1 = \frac{2\omega_p^2 - 8f_s^2}{k} \f]
    \f[ b_2 = \frac{\omega_p^2 + 4f_s^2 - 2\omega_pf_s/Q_p}{k} \f]

    then set correct gain

    \f[ a_i = a_i \omega_p^2 / \omega_z^2 \f]

    where

    \f[ k = 4f_s^2 + 2.0\omega_pf_s/Q_p + \omega_p^2 \f]

    \param	pf0  The pole frequency (Hz)
    \param	pq  The pole Q
    \param	zf0  The zero frequency (Hz)
    \param	zq  The zero Q
    \param  fs  The sample frequency of the filter
    \param  filt  A pointer to an miirfilter

    \retval 0
 */
int bilinear_complex_polezero(float pf0, float pq, float zf0, float zq, int fs, miirfilter *filt)
{
  double wp, wp2, wz, wz2, k;
  double a[3], b[3];

  wp = pf0*2*M_PIl;
  wp2 = wp*wp;
  wz = zf0*2*M_PIl;
  wz2 = wz*wz;

  /* Define coefficients */

  k = 4.0*fs*fs + 2.0*wp*fs/pq + wp2;

  a[0] = (4.0*fs*fs + 2.0*wz*fs/zq + wz2)/k;
  a[1] = (2.0*wz2 - 8.0*fs*fs)/k;
  a[2] = (4.0*fs*fs - 2.0*wz*fs/zq + wz2)/k;
  b[0] = 1.0;
  b[1] = (2.0*wp2 - 8.0*fs*fs)/k;
  b[2] = (wp2 + 4.0*fs*fs - 2.0*wp*fs/pq)/k;

  a[0] = a[0] * wp2 / wz2;
  a[1] = a[1] * wp2 / wz2;
  a[2] = a[2] * wp2 / wz2;

  /* build filter */

  sprintf(filt->name, "bcpz%3.2f-%3.2f-%d", pf0, zf0, fs);
  filt->fs = fs;
  filt->ntaps = 3;
  filt->dcgain = iirdcgain(a, b, filt->ntaps);
  filt->hfgain = iirhfgain(a, b, filt->ntaps);
  filt->gain = 1.0;

  filt->a[0] = a[0];
  filt->a[1] = a[1];
  filt->a[2] = a[2];
  filt->b[0] = b[0];
  filt->b[1] = b[1];
  filt->b[2] = b[2];

  return 0;
}



/*!
 *  Print a filter description to screen.
 *
 */
void dumpIIRfilt(miirfilter filt)
{
  int         i;

  // now print it out
  printf("--------------------------------------------------------------------------------\n");
  printf("NAME: \t\t%s\n", filt.name);
  printf("FS: \t\t%d\n", filt.fs);
  printf("NTAPS: \t\t%d\n", filt.ntaps);
  printf("GAIN: \t\t%2.10f\n", filt.gain);
  for(i=0; i<filt.ntaps; i++)
    printf("%d:\t\t%+05.20f\t\t%+05.20f\n", i, filt.a[i], filt.b[i]);
  for(i=0; i<filt.ntaps; i++)
    printf("%d:\t\t%+05.20f\t\t%+05.20f\n", i, filt.histin[i], filt.histout[i]);
  printf("--------------------------------------------------------------------------------\n");
}

/*!
 *  Print a filter description to file.
 *
 *  \param  filt The filter
 *  \param  filename The output filename
 *
 *  \retval 0 on success
 *  \retval -1 on error
 */
int dumpIIRfiltToFile(miirfilter filt, char *filename)
{
  FILE      *fid;
  int        i;
  time_t     now;

  if((fid = fopen(filename, "w+"))==NULL)
  {
    fprintf(stderr, "### dumpIIRfiltToFile: error opening file %s\n", filename);
    return -1;
  }

  printf("*** dumpFIRfiltToFile: dumping %s\n", filename);

  now = time(NULL);

  // now print it out
  fprintf(fid, "#---------------------------------------------------------------\n");
  fprintf(fid, "# Written by dumpIIRfiltToFile() \n");
  fprintf(fid, "# \n");
  fprintf(fid, "# \n");
  fprintf(fid, "# %s", ctime(&now));
  fprintf(fid, "#---------------------------------------------------------------\n");
  fprintf(fid, "NAME \t\t%s\n", filt.name);
  fprintf(fid, "NTAPS \t\t%d\n", filt.ntaps);
  fprintf(fid, "GAIN \t\t%2.10f\n", filt.gain);
  fprintf(fid, "FS \t\t%d\n", filt.fs);
  for(i=0; i<filt.ntaps; i++)
    fprintf(fid, "COEFF \t%05.20g \t%05.20g\n", filt.a[i], filt.b[i]);
  fprintf(fid, "\n");
  fprintf(fid, "\n");
  fprintf(fid, "#---------------------------------------------------------------\n");

  fclose(fid);

  return 0;
}


/*!
 * Read in a filter description from file into
 * an mfilter structure.
 *
 */
int readIIRfilt(char *filename, miirfilter *filt)
{
  FILE        *in;
	char         lbuffer[500];		    // line buffer for parsing
  const char   delims[] = " \t\n";
  char        *token;
  int          n, ncoeffs;
  char         norminst[5];

  printf("*** readIIRfilt: reading %s\n", filename);

  // open file
  in = fopen(filename, "r");
  if(in == NULL)
  {
    fprintf(stderr, "### readfilt: error opening file.\n");
    perror(filename);
    return -1;
  }

  // parse contents
  n = ncoeffs = 0;
  norminst[0] = '\0';

	while (fgets (lbuffer, 500, in) != NULL)
  {
    if(strlen(lbuffer) > 1)
    {
      // read the first token on this line
      token = strtok (lbuffer, delims);

      // ignore comments
      if(token[0] != '#')
      {
        if(strcmp(token, "NAME") == 0) // name tag
        {
          // get next token as name
          token = strtok (NULL, delims);
          if(token != NULL)
            strncpy(filt->name, token, 19);
        }
        else if(strcmp(token, "GAIN") == 0)
        {
          // get next token as gain
          token = strtok (NULL, delims);
          if(token != NULL)
          {
            filt->gain = atof(token);
            if(filt->gain < 0 || filt->gain > 1e20)
            {
              fprintf(stderr, "### readfilt: gain is a strange number (%2.10f)\n", filt->gain);
              return -1;
            }
          }
        }
        else if(strcmp(token, "FS") == 0)
        {
          // get next token as fs
          token = strtok (NULL, delims);
          if(token != NULL)
          {
            filt->fs = atoi(token);
            if(filt->fs < 0 || filt->fs > 1e10)
            {
              fprintf(stderr, "### readfilt: sample rate is a strange number (%d)\n", filt->fs);
              return -1;
            }
          }
        }
        else if(strcmp(token, "NTAPS") == 0)
        {
          // get next token as ntaps
          token = strtok (NULL, delims);
          if(token != NULL)
          {
            filt->ntaps = atoi(token);
            if(filt->ntaps <= 0 || filt->ntaps > MAX_TAPS)
            {
              fprintf(stderr, "### readfilt: number of taps is out of bounds (1->%d)\n", MAX_TAPS);
              return -1;
            }
          }
        }
        else if(strcmp(token, "NORM") == 0)
        {
          // get next token as normalisation instruction
          token = strtok (NULL, delims);
          strncpy(norminst, token, 5);
        }
        else if(strcmp(token, "COEFF") == 0)
        {
          if(ncoeffs < filt->ntaps)
          {
            // get next token as A coeff
            token = strtok (NULL, delims);
            if(token == NULL)
              filt->a[ncoeffs] = 0;
            else
              filt->a[ncoeffs] = atof(token);

            // get next token as B coeff
            token = strtok (NULL, delims);
            if(token == NULL)
              filt->b[ncoeffs] = 0;
            else
              filt->b[ncoeffs] = atof(token);

            ncoeffs++;
          }
          else
            fprintf(stderr, "### readfilt: max taps reached. Ignoring line %d\n", n);
        }
        else
          printf("*** readfilt: keyword not found (line %d).\n", n);

      }
    }

    n++;
  }

  // close file
  fclose(in);

  //printf("%s: norminst: %s\n", filt->name, norminst);

  // before returning, normalise gain if required
  if(strlen(norminst) > 0)
  {
    if(strcmp(norminst, "dc")==0)
    {
       // normalise dc to unity
       double gain = iirdcgain(filt->a, filt->b, filt->ntaps);
       int i;

       //printf("%s normalising dc\n", filt->name);
       for(i=0; i<filt->ntaps; i++)
         filt->a[i] /= gain;
    }
    else if(strcmp(norminst, "hf")==0)
    {
       // normalise hf to unity
       double gain = iirhfgain(filt->a, filt->b, filt->ntaps);
       int i;

       //printf("%s normalising hf\n", filt->name);
       for(i=0; i<filt->ntaps; i++)
         filt->a[i] /= gain;
    }
    else
    {
      fprintf(stderr, "### readfilt: normalisation instruction (%s) not recognised.\n", norminst);
      return -1;
    }
  }

  printf("*** done.\n");

  return 0;
}



/*! Calculate the DC gain of an IIR filter.
 *
 *  \param  double *a - the numerator filter taps
 *  \param  double *b - the denominator filter taps
 *  \param  int    na - the number of filter taps
 *
 *  \retval gain on success
 */
 INLINE double iirdcgain(double *a, double *b, int ntaps)
{
  int     i;
  double  g;
  double  suma, sumb;

  suma = sumb = 0.0;

  for(i=0; i<ntaps; i++)
  {
    suma += a[i];
    sumb += b[i];
  }

  g = suma / sumb;

  return g;
}

/*! Calculate the high frequency gain of an IIR filter.
 *
 *  \param  double *a - the numerator filter taps
 *  \param  double *b - the denominator filter taps
 *  \param  int    na - the number of filter taps
 *
 *  \retval gain on success
 */
 INLINE double iirhfgain(double *a, double *b, int ntaps)
{
  int     i;
  double  g;
  double  suma, sumb;

  suma = sumb = 0.0;

  for(i=0; i<ntaps; i++)
  {
    suma += a[i] * pow(-1.0, i-1);
    sumb += b[i] * pow(-1.0, i);
  }

  g = suma / sumb;

  return g;
}


/*! Normalise an IIR filter by the supplied gain.
 *
 *  \param  double *a - the numerator filter taps
 *  \param  int    na - the number of filter taps
 *  \param  double  g - the gain
 *
 *  \retval 0 on success
 */
 INLINE int iirnormalise(double *a, int na, double g)
{
  int i;

  for(i=0; i<na; i++)
    a[i] /= g;

  return 0;
}
